use anyhow::Result;
use async_graphql::{InputObject, SimpleObject};
use serde::{Deserialize, Serialize};
use sqlx::{FromRow, PgPool, Type};

#[derive(SimpleObject, Debug, FromRow, Deserialize, Serialize, Type)]
pub struct Typ {
    pub id: i32,

    /// Name eines Typs
    pub name: String,
}

#[derive(InputObject, Debug)]
pub struct TypCreateInput {
    pub name: String,
}

#[derive(InputObject, Debug)]
pub struct TypUpdateInput {
    pub id: i32,
    pub name: Option<String>,
}

impl Typ {
    pub async fn read_one(pool: &PgPool, id: &i32) -> Result<Typ> {
        let row = sqlx::query_as!(Typ, "SELECT * FROM typen WHERE id = $1", id)
            .fetch_one(pool)
            .await?;

        Ok(row)
    }

    pub async fn read_all(pool: &PgPool) -> Result<Vec<Typ>> {
        let rows = sqlx::query_as!(Typ, "SELECT id, name FROM typen")
            .fetch_all(pool)
            .await?;

        Ok(rows)
    }

    pub async fn create(pool: &PgPool, input: TypCreateInput) -> Result<Typ> {
        let row = sqlx::query!(
            "INSERT INTO typen(name) VALUES ($1) RETURNING id",
            input.name
        )
        .fetch_one(pool)
        .await?;

        let result = Self::read_one(pool, &row.id).await?;

        Ok(result)
    }

    pub async fn create_many(pool: &PgPool, input: &[TypCreateInput]) -> Result<Typ> {
        let mut v1: Vec<&str> = Vec::new();
        input.iter().for_each(|typ| v1.push(&typ.name));

        let row = sqlx::query_as(
            r#"
            INSERT INTO typen(name)
            SELECT * FROM UNNEST($1)
            RETURNING id, name"#,
        )
        .bind(&v1)
        .fetch_one(pool)
        .await?;

        Ok(row)
    }

    pub async fn update(pool: &PgPool, input: TypUpdateInput) -> Result<Typ> {
        sqlx::query!(
            "UPDATE typen SET name=$1 WHERE id = $2",
            input.name,
            input.id
        )
        .execute(pool)
        .await?;

        let t = Typ::read_one(pool, &input.id).await?;
        Ok(t)
    }

    pub async fn delete(pool: &PgPool, id: &i32) -> Result<bool> {
        let result = sqlx::query!("DELETE FROM typen WHERE id = $1", id)
            .execute(pool)
            .await?;

        Ok(result.rows_affected() > 0)
    }
}
